<?php

use Models\WebServices\Mapper\Mapper as Mapper;

class VMwareManager {

    protected $mapper;

    public function __construct(Mapper $mapper = null)
    {
        $this->mapper = $mapper ?: new Mapper();
    }


    public $instpath = "/usr/local/opmon/libexec/modelgroups/vmwareguest/agents";
    private $timeToCache = 300;

	private function get_basic_params($data) {
        if (!isset($data->globalserver) || empty($data->globalserver))
            throw new Exception("Please provide server ip adress");
        if (!isset($data->globalportnumber) || empty($data->globalportnumber))
            throw new Exception("Please provide server port number");
        if (!isset($data->globalusername) || empty($data->globalusername))
            throw new Exception("Please provide server username");
        if (!isset($data->globalpassword) || empty($data->globalpassword))
            throw new Exception("Please provide server username's password");

        $params = sprintf(
            "%s --server %s ",
            $params,
            escapeshellarg($data->globalserver)
        );

        $params = sprintf(
            "%s --portnumber %s ",
            $params,
            escapeshellarg($data->globalportnumber)
        );

        $params = sprintf(
            "%s --username %s ",
            $params,
            escapeshellarg($data->globalusername)
        );

        $params = sprintf(
            "%s --password %s ",
            $params,
            escapeshellarg($data->globalpassword)
        );

        $params = sprintf(
            "%s --managedobject %s ",
            $params,
            "VirtualMachine"
        );
        return $params;
    }

    private function runCommand(ShellCommandRunner $runner, $cmd)
    {
        $runner->run($cmd);

        if ($runner->get_stderr())
            throw new Exception($runner->get_stderr());

        $ret = json_decode($runner->get_stdout());

        if (json_last_error() != JSON_ERROR_NONE)
            throw new Exception('Invalid return from VMware SDK');

        if (! is_object($ret) || !isset($ret->success))
            throw new Exception('Could not interpret feedback received from VMware SDK');

        if (! $ret->success)
            throw new Exception($ret->message);

        if (! $ret->data)
            throw new Exception("There was no return from VMware SDK");

        return $ret->data;
    }

	private function list_managed_object_counters($data, $counters, $parameterName) {
		$data->counters = $counters;
		$basic_params = $this->get_basic_params($data);
		if (!isset($data->globalguest) || empty($data->globalguest)) {
			throw new Exception('Provide a name to retrieve counters');
		}

		$mem_idx = $this->mount_request_memcache_index($data);
		$on_mem = OPCache::get($mem_idx);
		if ($on_mem !== FALSE && ! empty($on_mem)) {
			return $on_mem;
		}

		$list = array();
		$cmd = sprintf(
			"%s/manager/scripts/get_managed_objects_counters.pl %s --name '%s'",
			$this->instpath,
			$basic_params,
			$data->globalguest
		);

		$runner = new ShellCommandRunner();
		$output = $this->runCommand($runner, $cmd);
		$metrics = array_filter($output, function($metric) use ($counters) {
			if (in_array($metric->key, $counters)) {
				return $metric;
			}
		});

		foreach ($metrics as $metric) {
			if (in_array($metric, $list)) {
				continue;
			}

			$cmd = sprintf(
				"%s/check_vmware_counter.pl %s --name '%s' --metric %s",
				$this->instpath,
				$basic_params,
				$guest,
				$metric->value
			);

			$runner->run($cmd);
			if (! $runner->get_stdout()) {
				continue;
			}
			$list[] = $metric;
			$this->mapper->addElement(
				$metric->key,
				$metric->key,
				array(
					$parameterName => $metric->value
				)
			);
		}
		$list = array_map(function($item) {
			return array('key' => $item->key, 'value' => $item->value);
		}, $list);

		if (empty($list)) {
			throw new Exception('This service not available');
		}

		foreach ($list as $result) {
			$this->mapper->addElement(
				$result['key'],
				$result['value'],
				array(
					"cpu_percent" => $result['key'],
					"globalserver" => $data->globalserver,
					"globalportnumber" => $data->globalportnumber,
					"globalusername" => $data->globalusername,
					"globalpassword" => $data->globalpassword
				)
		    );

		}



		$this->mapper->setComboData($list);
		$results = $this->mapper->getOutput();

		OPCache::set($mem_idx, $results, $this->timeToCache);
		return $results;
	}

    public function get_vm_cpu_percent_counters($data)
    {
        $counters = array(
            'CPU Latency',
            'CPU Usage',
            'Resource group CPU Running (1 min peak)',
            'Resource group CPU Running (5 min peak)',
            'Resource group CPU Active (5 min peak)',
            'CPU Readiness',
            'Resource group CPU Active (5 min average)',
            'CPU Demand-to-entitlement ratio',
            'Resource group CPU Running (1 min average)',
            'Resource group CPU Active (1 min peak)',
            'Resource group CPU Active (1 min average)',
            'Resource group CPU Throttled (1 min average)',
            'Resource group CPU Running (15 min average)',
            'Resource group CPU Active (15 min average)',
            'Resource group CPU Throttled (15 min average)',
            'Resource group CPU Running (15 min peak)',
            'Resource group CPU Active (15 min peak)',
            'Resource group CPU Throttled (5 min average)',
            'Resource group CPU Running (5 min average)'
		);

        return $this->list_managed_object_counters($data, $counters, "cpu_percent");
    }

    public function get_vm_cpu_ms_counters($data)
    {
        $counters = array(
            'CPU Wait',
            'CPU Overlap',
            'CPU System',
            'CPU Used',
            'Resource group CPU Group CPU sample period',
            'CPU Ready',
            'CPU Run',
            'CPU Idle',
            'CPU Co-stop',
            'CPU Max limited',
            'CPU Swap wait'
        );
        return $this->list_managed_object_counters($data, $counters, "cpu_ms");
    }

    public function get_vm_cpu_htz_counters($data)
    {
        $counters = array(
            'CPU Demand',
            'CPU Usage in MHz',
            'CPU Entitlement'
        );
        return $this->list_managed_object_counters($data, $counters, "cpu_htz");
    }

    public function get_vm_disk_ms_counters($data)
    {
        $counters = array(
            'Disk Highest latency',
            'Virtual disk Read latency',
            'Virtual disk Write latency',
            'Virtual disk Read Latency (us)',
            'Virtual disk Write Latency (us)'
        );
        return $this->list_managed_object_counters($data, $counters, "disk_ms");
    }

    public function get_vm_disk_kbps_counters($data)
    {
        $counters = array(
            'Disk Usage',
            'Disk Read rate',
            'Disk Write rate',
            'Virtual disk Read rate',
            'Virtual disk Read rate',
            'Virtual disk Read request size',
            'Virtual disk Write request size'
        );
        return $this->list_managed_object_counters($data, $counters, "disk_kbps");
    }

    public function get_vm_disk_number_counters($data)
    {
        $counters = array(
            'Disk Read requests',
            'Disk Write requests',
            'Disk Commands issued',
            'Disk Commands aborted',
            'Disk Bus resets',
            'Disk Average read requests per second',
            'Disk Average write requests per second',
            'Disk Average commands issued per second',
            'Virtual disk Average read requests per second',
            'Virtual disk Average write requests per second',
            'Virtual disk Average number of outstanding read requests',
            'Virtual disk Average number of outstanding write requests',
            'Virtual disk Read workload metric',
            'Virtual disk Write workload metric',
            'Virtual disk Number of small seeks',
            'Virtual disk Number of medium seeks',
            'Virtual disk Number of large seeks'
        );
        return $this->list_managed_object_counters($data, $counters, "disk_number");
    }

    public function get_vm_memory_percent_counters($data)
    {
        $counters = array(
            'Memory Host consumed %',
            'Memory Page-fault latency'
        );
        return $this->list_managed_object_counters($data, $counters, "memory_percent");
    }

    public function get_vm_memory_kb_counters($data)
    {
        $counters = array(
            'Memory Granted',
            'Memory Active',
            'Memory Shared',
            'Memory Zero pages',
            'Memory Ballooned memory',
            'Memory Overhead consumed',
            'Memory Swapped',
            'Memory Swap target',
            'Memory Swap in',
            'Memory Swap out',
            'Memory Balloon target',
            'Memory Consumed',
            'Memory Active write',
            'Memory Compressed',
            'Memory Overhead reserved',
            'Memory Compressed',
            'Memory Compression saved',
            'Memory Entitlement',
            'Memory Overhead active',
            'Memory Host cache consumed'
        );
        return $this->list_managed_object_counters($data, $counters, "memory_kb");
    }

    public function get_vm_memory_kbps_counters($data)
    {
        $counters = array(
            'Memory Swap in rate',
            'Memory Swap out rate',
            'Memory Compression rate',
            'Memory Decompression rate',
            'Memory Host cache swap in rate',
            'Memory Host cache swap out rate'
        );
        return $this->list_managed_object_counters($data, $counters,"memory_kbps");
    }

    public function get_vm_network_kbps_counters($data)
    {
        $counters = array(
            'Network Usage',
            'Network Data receive rate',
            'Network Data transmit rate',
            'Network pnicBytesRx',
            'Network pnicBytesTx'
        );
        return $this->list_managed_object_counters($data, $counters, "network_kbps");
    }

    public function get_vm_network_number_counters($data)
    {
        $counters = array(
            'Network Packets received',
            'Network Packets transmitted',
            'Network Receive packets dropped',
            'Network Transmit packets dropped',
            'Network Broadcast receives',
            'Network Broadcast transmits',
            'Network Multicast receives',
            'Network Multicast transmits'
        );
        return $this->list_managed_object_counters($data, $counters, "network_number");
    }

    public function get_vm_datastore_ms_counters($data)
    {
        $counters = array(
            'Datastore Highest latency',
            'Datastore Read latency',
            'Datastore Write latency'
        );
        return $this->list_managed_object_counters($data, $counters, "datastore_millisecond");
    }

    public function get_vm_datastore_number_counters($data)
    {
        $counters = array(
            'Datastore Average read requests per second',
            'Datastore Average write requests per second'
        );
        return $this->list_managed_object_counters($data, $counters, "datastore_number");
    }

    public function get_vm_datastore_kbps_counters($data)
    {
        $counters = array(
            'Datastore Write rate',
            'Datastore Read rate'
        );
        return $this->list_managed_object_counters($data, $counters, "datastore_kbps");
    }

    public function get_vm_system_counters($data)
    {
        $counters = array(
            'System Heartbeat',
            'System OS Uptime',
            'System Uptime'
        );
        return $this->list_managed_object_counters($data, $counters, "system");
    }

	public function get_vms($data)
	{
		$basic_params = $this->get_basic_params($data);

        $cmd = sprintf(
            "%s/manager/scripts/list_managed_objects.pl %s ",
            $this->instpath,
            $basic_params
		);
		$runner = new ShellCommandRunner();
		$values = $this->runCommand($runner, $cmd);

		foreach ($values as $output) {
			$this->mapper->addElement(
				$output->key,
				$output->key,
				array(
					"globalguest" => $output->key,
					"globalserver" => $data->globalserver,
					"globalportnumber" => $data->globalportnumber,
					"globalusername" => $data->globalusername,
					"globalpassword" => $data->globalpassword
				),
				true
		    );

		}

		$this->mapper->setComboData($values);
        return $this->mapper->getOutput();
    }

    private function mount_request_memcache_index($data) {
        $dtstr = serialize($data);
        return md5($dtstr);
    }
}


